/*--------------------------------------------------------------------------
 * Copyright (C) 1992-1999 Dallas Semiconductor Corporation. 
 * All rights Reserved. Printed in U.S.A.
 * This software is protected by copyright laws of
 * the United States and of foreign countries.
 * This material may also be protected by patent laws of the United States 
 * and of foreign countries.
 * This software is furnished under a license agreement and/or a
 * nondisclosure agreement and may only be used or copied in accordance
 * with the terms of those agreements.
 * The mere transfer of this software does not imply any licenses
 * of trade secrets, proprietary technology, copyrights, patents,
 * trademarks, maskwork rights, or any other form of intellectual
 * property whatsoever. Dallas Semiconductor retains all ownership rights.
 *--------------------------------------------------------------------------
 *
 *  MonDec.C - This utility uses TMEX decrement the money file on a 
 *             DS1963. 
 *
 *  Compiler:  Microsoft Visual C 
 *  Version:   1.00
 *
 */

#define TMEXUTIL

#include "ibtmex.h"   

/* external function prototypes */
extern short ReadDefaultPort(void);
extern void Exit_Prog(char *, short);
extern void PrintHeader(char *);
extern short Aquire1WireNet(void);
extern void Release1WireNet(void);
extern short WaitForiButton(uchar);
extern short InterruptOperation(void);
extern short EnterNum(char *, short, ulong *);
extern short EnterStr(char *, short, char *);
extern short ChangeMoneyBalance(ulong, ulong, FileEntry *);
extern short VerifySignedMoney(ulong *, FileEntry *, uchar *, short);
extern short ReadMoneyFiles(uchar *, short *, uchar *, short *, FileEntry *);

/*----------------------------------------------------------------------
 *  This is the Main routine for MonDec
 */
void main(short argc, char **argv)
{
   short moneyok;
   short money_len,info_len,st;
   ulong balance,dec;
   uchar money_buf[500],info_buf[500];
   FileEntry MoneyFile;
  
   /* check arguments to see if request instruction with '?' or too many */
   if (argc > 1)
   {
       printf("\nusage: MonDec\n"
              "  - decrement money file\n"
              "  - version 1.00\n");
       Exit_Prog("",BAD_ARGUMENT);
   }

   /* read the default 1-Wire Net Port */
   if (!ReadDefaultPort())
     Exit_Prog("ERROR, Could not read the default PortNum and PortType from registry!\n",
               NO_DRIVERS);

   /* print the header info */
   PrintHeader("MonDec Version 1.00");

   /* enter the info, balance */
   if (EnterNum("Enter amount to decrement (0 to 4000000000): ", 
             10, &dec))
   {
      /* aquired the 1-Wire Net */
      printf("\nAquire a session on the 1-Wire Net...\n");
      if (Aquire1WireNet())
      {
         printf("STATUS:%s\n",LastStatus);
         printf("\nWait for a Monetary iButton...\n");

         /* wait for a monetary iButton to arrive */
         if (WaitForiButton(MONETARY_FAM))
         {      
            printf("STATUS:%s\n",LastStatus);

            /* read both files */
            if (ReadMoneyFiles(info_buf, &info_len, money_buf, 
                 &money_len, &MoneyFile))
            {
               printf("STATUS:%s\n",LastStatus);
            
               /* assume info is text, put in status */
               info_buf[info_len] = 0;
               st = sprintf(LastStatus,"\n\nInformation File Contents:\n%s\n\n",(char *)info_buf);

               st += sprintf(&LastStatus[st],"Money File Contents:\n");
               moneyok = VerifySignedMoney(&balance, &MoneyFile, money_buf, money_len);
               if (moneyok)
                  st += sprintf(&LastStatus[st],"----- VERIFIED -----");
               else
                  st += sprintf(&LastStatus[st],"?????? INVALID ?????");
               st += sprintf(&LastStatus[st],"\nBalance: %d\n\n",balance);
         
               /* if money is ok then deduct */
               if (moneyok)
               {
                  printf("STATUS:%s\n",LastStatus);

                  /* check for insufficient funds */
                  if (dec > balance) 
                     sprintf(LastStatus,"Money file contains insufficient funds for this sale!\n\n");
                  else                  
                  {
                     if (ChangeMoneyBalance(balance,balance - dec,&MoneyFile))
                        sprintf(LastStatus,"Success, balance is now %d\n\n",balance - dec);
                     else
                        sprintf(LastStatus,"Error deducting money!\n\n");
                  }
               }  
               else
                  st += sprintf(&LastStatus[st],"\nMoney file is not valid so cannot deduct amount!\n\n");
            }
            else
               sprintf(LastStatus,"Read aborted!\n");
         }
         else
           sprintf(LastStatus,"Wait for Monetary iButton aborted!\n");
      
      }
      else
         sprintf(LastStatus,"Wait for 1-Wire Net aborted!\n");
   }
   else
     sprintf(LastStatus,"Data entry aborted!\n");

   printf("STATUS:%s\n",LastStatus);

   /* release the 1-Wire Net to other processes */
   Release1WireNet();
   printf("STATUS:%s\n",LastStatus);

   /* normal exit */
   Exit_Prog("", NORMAL_EXIT);
}