/*--------------------------------------------------------------------------
 * Copyright (C) 1992-1999 Dallas Semiconductor Corporation. 
 * All rights Reserved. Printed in U.S.A.
 * This software is protected by copyright laws of
 * the United States and of foreign countries.
 * This material may also be protected by patent laws of the United States 
 * and of foreign countries.
 * This software is furnished under a license agreement and/or a
 * nondisclosure agreement and may only be used or copied in accordance
 * with the terms of those agreements.
 * The mere transfer of this software does not imply any licenses
 * of trade secrets, proprietary technology, copyrights, patents,
 * trademarks, maskwork rights, or any other form of intellectual
 * property whatsoever. Dallas Semiconductor retains all ownership rights.
 *--------------------------------------------------------------------------
 *
 *  MonInit.C - This utility uses TMEX to initialize DS1963 information
 *              and money file. 
 *
 *  Compiler:  Microsoft Visual C 
 *  Version:   1.00
 *
 */

#define TMEXUTIL

#include "ibtmex.h"   

/* external function prototypes */
extern short ReadDefaultPort(void);
extern void Exit_Prog(char *, short);
extern void PrintHeader(char *);
extern short Aquire1WireNet(void);
extern void Release1WireNet(void);
extern short WaitForiButton(uchar);
extern short MonetaryiButtonNotFormated(void);
extern short InterruptOperation(void);
extern short EnterNum(char *, short, ulong *);
extern short EnterStr(char *, short, char *);
extern short InitMonetaryiButton(ulong, char *);

/*----------------------------------------------------------------------
 *  This is the Main routine for MonInit
 */
void main(short argc, char **argv)
{
   ulong balance;
   uchar info_buf[500];

   /* check arguments to see if request instruction with '?' or too many */
   if (argc > 1)
   {
       printf("\nusage: MonInit\n"
              "  - initialize the info and money file\n"
              "  - version 1.00\n");
       Exit_Prog("",BAD_ARGUMENT);
   }

   /* read the default 1-Wire Net Port */
   if (!ReadDefaultPort())
     Exit_Prog("ERROR, Could not read the default PortNum and PortType from registry!\n",
               NO_DRIVERS);

   /* print the header info */
   PrintHeader("MonInit Version 1.00");

   /* enter the info, balance */
   if (EnterNum("Enter the starting balance (0 to 4000000000): ", 
             10, &balance) &&
       EnterStr("\nEnter the personal information (business card): ", 
             60, info_buf))
   {
      /* aquired the 1-Wire Net */
      printf("\nAquire a session on the 1-Wire Net...\n");
      if (Aquire1WireNet())
      {
         printf("STATUS:%s\n",LastStatus);
         printf("\nWait for a Monetary iButton...\n");

         /* wait for a monetary iButton to arrive */
         if (WaitForiButton(MONETARY_FAM))
         {      
            printf("STATUS:%s\n",LastStatus);

            /* check if formated */
            if (!MonetaryiButtonNotFormated())
               printf("\nWarning this device is currently formated\n\n");
         
            /* format, write two files */
            if (!InitMonetaryiButton(balance,info_buf))
               printf("Initialization aborted!\n");
         }
         else
           printf("Wait for Monetary iButton aborted!\n");
      
      }
      else
        printf("Wait for 1-Wire Net aborted!\n");
   }
   else
     printf("Data entry aborted!\n");

   printf("STATUS:%s\n",LastStatus);

   /* release the 1-Wire Net to other processes */
   Release1WireNet();
   printf("STATUS:%s\n",LastStatus);

   /* normal exit */
   Exit_Prog("", NORMAL_EXIT);
}