/*--------------------------------------------------------------------------
 * Copyright (C) 1992-1999 Dallas Semiconductor Corporation. 
 * All rights Reserved. Printed in U.S.A.
 * This software is protected by copyright laws of
 * the United States and of foreign countries.
 * This material may also be protected by patent laws of the United States 
 * and of foreign countries.
 * This software is furnished under a license agreement and/or a
 * nondisclosure agreement and may only be used or copied in accordance
 * with the terms of those agreements.
 * The mere transfer of this software does not imply any licenses
 * of trade secrets, proprietary technology, copyrights, patents,
 * trademarks, maskwork rights, or any other form of intellectual
 * property whatsoever. Dallas Semiconductor retains all ownership rights.
 *--------------------------------------------------------------------------
 *
 *  MonUtils.C - Monetary iButton utility functions 
 *
 *  Compiler:  Microsoft Visual C 
 *  Version:   1.00
 *
 */

#include "ibtmex.h"   
#include "secret.h"
#include "verf.h"

/* Local Function Prototypes */
short VerifySignedMoney(ulong *, FileEntry *, uchar *, short);
void CreateSignedMoney(ulong, FileEntry *, uchar *);
short iBWriteMoneyFile(FileEntry *, ulong, short);
short MonetaryiButtonNotFormated(void);
short InitMonetaryiButton(ulong, char *);
short ReadMoneyFiles(uchar *, short *, uchar *, short *, FileEntry *);
void CalculateSignature(uchar *, FileEntry *, uchar *);
short ChangeMoneyBalance(ulong, ulong, FileEntry *);

/* external function prototypes */
extern short VerifyCRCPacket(short, uchar *);
extern short iBWriteFile(FileEntry *, uchar *, short);
extern short InterruptOperation(void);
extern short iBReadFile(FileEntry *, uchar *, short *);
extern uchar iBGetStartPage(FileEntry *);

/*----------------------------------------------------------------------
 * Check if device found is not formated
 * return TRUE(1)  : device verified to NOT be formated 
 *        FALSE(0) : device formated or error reading device
 */                                                          
short MonetaryiButtonNotFormated(void)
{
   uchar read_buf[50];  

   /* attempt to read the first page */
   if (TMExtendedReadPage(session_handle,state_buffer,0,read_buf,0) == 40)
   {
      /* check for valid data packet */
      if (VerifyCRCPacket(0,read_buf))
      {
         /* make sure length - 1 is mod 7, and majic bytes correct */
         if ((read_buf[0] > 0) && (((read_buf[0] - 1) % 7) == 0) &&
             (read_buf[1] == 0xAA) && (read_buf[2] == 0)) 
         {
            sprintf(LastStatus,"Device is formated");
            return FALSE;
         }
         else
            sprintf(LastStatus,"Device has valid packet but is not a directory");
      }      
      else
         sprintf(LastStatus,"Device does not have a valid packet where directory should be");
   }
   else
   {
      sprintf(LastStatus,"Error reading directory page, poor contact");
      return FALSE;  
   }

   /* device not formated */
   return TRUE;
}


/*----------------------------------------------------------------------
 * Write the specified money file.  Take the specified balance and
 * create the correct signature and write the file.
 * The 'newfile' flag indicates that the file is new (1) or existing (0)
 * with the current file options in the flname structure.
 * return TRUE(1)  : File written successfully
 *        FALSE(0) : Error reading file
 */                                                          
short iBWriteMoneyFile(FileEntry *flname, ulong balance, short newfile)
{
   short file_handle,maxwrite,rslt;
   uchar buf[50];

   /* if new file then create it */
   if (newfile)
   {
      /* delete the file first */
      rslt = TMDeleteFile(session_handle,state_buffer,flname);
      /* ok if succes or file already deleted */
      if ((rslt == 1) || (rslt == XFILE_NOT_FOUND))
      {
         /* close any opened files */
         for (file_handle = 0; file_handle < 10; file_handle++)
            TMCloseFile(session_handle,state_buffer,file_handle);

         /* create the file */
         file_handle = TMCreateFile(session_handle,state_buffer,&maxwrite,flname);

         /* check if valid handle */
         if (file_handle >= 0)
         {
            /* create the file data */
            CreateSignedMoney(balance,flname,buf);

            /* proceed with write */
            rslt = TMWriteFile(session_handle,state_buffer,file_handle,
                                buf,28);
            TMCloseFile(session_handle,state_buffer,file_handle);
            /* check results */
            if (rslt > 0)
            {
               sprintf(LastStatus,"File written successfully");
               return TRUE;
            }
            else
            {
               sprintf(LastError,"Error writing file %d",rslt); 
               sprintf(LastStatus,"%s",LastError);
               return FALSE;
            }   
         }
         else
         {
            sprintf(LastError,"Error creating file %d",file_handle); 
            sprintf(LastStatus,"%s",LastError);
            return FALSE;
         }   
      }
      else
      {
         sprintf(LastError,"Error deleting file %d",rslt); 
         sprintf(LastStatus,"%s",LastError);
         return FALSE;
      }   
   }
   /* existing file, only replace the page data */
   else 
   {
      /* create the file data */
      CreateSignedMoney(balance,flname,buf);
      /* add the 0 page pointer */
      buf[28] = 0;      

      /* write that page only */
      rslt = TMWritePacket(session_handle,state_buffer,flname->startpage,
                           buf,29);
      if (rslt == 29)
      {
         /* success, verify????? */
         sprintf(LastStatus,"File updated successfully");
         return TRUE;
      }   
      else
      {
         sprintf(LastError,"Error updating the money file %d",rslt); 
         sprintf(LastStatus,"%s",LastError);
         return  FALSE; 
      }
   }
}


/*----------------------------------------------------------------------
 * Create the signed monetary file from the info provided.
 */                                                          
void CreateSignedMoney(ulong balance, FileEntry *flname, uchar *buf)
{
   short bufcnt=0,i;
   ulong tlong;

   /* put the money balance in the file */
   for (i = 0; i < 4; i++)
   {
      buf[bufcnt++] = (uchar)(balance & 0xFF);      
      balance >>= 8;
   }
   
   /* clear the next 3 bytes for future expansion */
   for (i = 0; i < 3; i++)
      buf[bufcnt++] = 0;

   /* set the transaction host */
   buf[bufcnt++] = HOST_ID;

   /* increment the tranaction count and put back in file entry */
   /* this is the FUTURE value of the count after copying */
   /* extract from file entry */
   tlong = 0; 
   for (i = 3; i >= 0; i--)
   {
      tlong <<= 8;
      tlong |= flname->bitmap[i];      
   }
   /* increment and then put back in */
   tlong++;
   for (i = 0; i < 4; i++)
   {
      flname->bitmap[i] = (uchar)(tlong & 0xFF);      
      tlong >>= 8;
   }
      
   /* calculate the signature and place in buf */
   CalculateSignature(buf, flname, &buf[bufcnt]);
}


/*----------------------------------------------------------------------
 * Calculate the signature from:
 *     SecretTop              (12)  (from secret.h)
 *     New Transaction count  (4)   (in flname bitmap bytes 4 to 7)
 *     Money & host ID        (8)   (data that can be read in clear)
 *     Registration Number    (8)   (ID number of DS1963)
 *     Money file page number (1)   (in flname startpage)
 *     SecretBottom           (12)  (from secret.h)
 */                                                          
void CalculateSignature(uchar *clear_data, FileEntry *flname, uchar *signature)
{
   short hlen=0,i,j;
   uchar hashbuf[500];
   ulong tlong; 
   Hash  Digest;
  
   /* create the data to hash */
   /* top secret */
   for (i = 0; i < 12; i++)
      hashbuf[hlen++] = SecretKeyTop[i];
   /* transaction count */
   for (i = 0; i < 4; i++)
      hashbuf[hlen++] = flname->bitmap[i];      
   /* clear data (money and host ID) */
   for (i = 0; i < 8; i++)
      hashbuf[hlen++] = clear_data[i];
   /* registration number of device */
   for (i = 0; i < 8; i++)
      hashbuf[hlen++] = (uchar)ROM[i];
   /* page where money file is located */
   hashbuf[hlen++] = flname->startpage;
   /* bottom secret */
   for (i = 0; i < 12; i++)
      hashbuf[hlen++] = SecretKeyBottom[i];

   /* hash the data */
   HashMessage(hashbuf, hlen, Digest);

   /* copy the hash data into signature buffer */
   for (j = 0; j < 5; j++)
   {
      tlong = Digest[j];
      for (i = 0; i < 4; i++)
      {
         signature[j * 4 + i] = (uchar)(tlong & 0xFF);      
         tlong >>= 8;
      }
   }       
}


/*----------------------------------------------------------------------
 * Verify the signed monetary file from the info provided.
 * return TRUE(1)  : signed money is valid, balance is set
 *        FALSE(0) : signed money not valid
 */                                                          
short VerifySignedMoney(ulong *balance, FileEntry *flname, uchar *money_buf, short money_len)
{
   short bufcnt=0,i;
   uchar checksig[30];

   *balance = 0; 

   /* extract the money balance */
   for (i = 3; i >= 0; i--)
   {
      *balance <<= 8;
      *balance |= money_buf[i];      
   }

   /* verify the not used bytes are 0 */
   if ((money_buf[4] != 0) || (money_buf[5] != 0) || (money_buf[6] != 0)) 
      return FALSE;

   /* check tamper bytes */ 
   for (i = 0; i < 4; i++)
      if (money_buf[money_len + 4 + i] != 0x55)
         return FALSE;

   /* place the counter bytes into to file name structure 
      where calculate signature expects it */
   for (i = 0; i < 4; i++)
      flname->bitmap[i] = money_buf[money_len + i];

   /* check the hash */
   CalculateSignature(money_buf, flname, checksig);
   for (i = 0; i < 20; i++)
      if (checksig[i] != money_buf[i + 8])
         return FALSE;

   /* signature must be correct */   
   return TRUE;
}

/*----------------------------------------------------------------------
 * Format and write the two files for the Monetary iButton
 */                                                          
short InitMonetaryiButton(ulong balance, char *info_buf)
{
   short operation=0, errorcnt=0, printmsg=1,rslt;
   FileEntry MoneyFile,InfoFile;


   /* loop to do all three steps */
   do
   {
      switch (operation)
      {
         case 0:  /* format */
            if (TMFormat(session_handle, state_buffer) == 1)
            {
               /* success so go to next operation */
               sprintf(LastStatus,"Device formated");
               operation = 1;
               errorcnt = 0;
            } 
            else
               errorcnt++; 
            break;
         case 1:  /* write info */
            sprintf(&InfoFile.name[0],InfoName);
            InfoFile.extension = 0;
            if (iBWriteFile(&InfoFile,info_buf,(short)strlen(info_buf)))
            {
               /* success so go to next operation */
               sprintf(LastStatus,"Info file written");
               operation = 2;
               errorcnt = 0;
            } 
            else
               errorcnt++; 
            break;
         case 2:  /* write money */
            sprintf(&MoneyFile.name[0],MoneyName);
            MoneyFile.extension = 101;
            if (iBWriteMoneyFile(&MoneyFile,balance,TRUE))
            {
               sprintf(LastStatus,"Money file written");
               return TRUE;
            }
            else
               errorcnt++; 
            break;
      };
      
      /* check for an error, verify device is present */
      if (errorcnt > 0)      
      {
         rslt = TMStrongAccess(session_handle, state_buffer);
         if (rslt != 1)
         {
            if (printmsg)
            {
               printf("\nPlease reconnect the iButton to complete the initialization!\n");
               printmsg = FALSE; 
            }
            errorcnt = 0; 
         }
      }
      else
         printmsg = TRUE;

      /* check for too many errors */
      if (errorcnt > 40)
      {
         sprintf(LastError,"Failed to complete the initialization!\n");
         sprintf(LastStatus,"%s",LastError);
         return FALSE;
      } 
   }
   while (!InterruptOperation());

   sprintf(LastError,"Operation aborted\n");
   sprintf(LastStatus,"%s",LastError);

   return FALSE;
}


/*----------------------------------------------------------------------
 * Format and write the two files for the Monetary iButton
 */                                                          
short ReadMoneyFiles(uchar *info_buf, short *info_len,
                     uchar *money_buf, short *money_len, FileEntry *MoneyFile)
{
   short operation=0, errorcnt=0, printmsg=1,rslt;
   FileEntry InfoFile;

   /* loop to do both steps */
   do
   {
      switch (operation)
      {
         case 0:  /* read info */
            sprintf(&InfoFile.name[0],"INFO");
            InfoFile.extension = 0;
            if (iBReadFile(&InfoFile,info_buf,info_len))
            {
               /* success so go to next operation */
               sprintf(LastStatus,"Info file read");
               operation = 1;
               errorcnt = 0;
            } 
            else
               errorcnt++; 
            break;
         case 1:  /* read money file */
            sprintf(&MoneyFile->name[0],"MONY");
            MoneyFile->extension = 101;
            if (iBReadFile(MoneyFile,money_buf,money_len))
            {
               /* since this is a money file we will need the 
                  page number */
               MoneyFile->startpage = iBGetStartPage(MoneyFile);
               sprintf(LastStatus,"Money and Info file read");
               return TRUE;
            }
            else
               errorcnt++; 
            break;
      };
      
      /* check for an error, verify device is present */
      if (errorcnt > 0)      
      {
         rslt = TMStrongAccess(session_handle, state_buffer);
         if (rslt != 1)
         {
            if (printmsg)
            {
               printf("\nPlease reconnect the iButton to complete the read!\n");
               printmsg = FALSE; 
            }
            errorcnt = 0; 
         }
      }
      else
         printmsg = TRUE;

      /* check for too many errors */
      if (errorcnt > 40)
      {
         sprintf(LastError,"Failed to complete the read!\n");
         sprintf(LastStatus,"%s",LastError);
         return FALSE;
      } 
   }
   while (!InterruptOperation());

   sprintf(LastError,"Operation aborted\n");
   sprintf(LastStatus,"%s",LastError);

   return FALSE;
}


/*----------------------------------------------------------------------
 * Change the balance in the money file
 * 
 */                                                          
short ChangeMoneyBalance(ulong oldbalance, ulong newbalance, FileEntry *MoneyFile)
{
   short operation=0, errorcnt=0, printmsg=1, rslt, i;
   uchar read_buf[50];
   ulong balance;

   /* loop to do both steps */
   do
   {
      switch (operation)
      {
         case 0: /* write the new balance */
            if (iBWriteMoneyFile(MoneyFile,newbalance,FALSE))
            {
               sprintf(LastStatus,"Money file updated");
               errorcnt = 0; 
               operation = 1;
            }
            else
            {
               /* count the error but proceed to verify to get
                  latest counter values */
               errorcnt++;
               operation = 1;
            } 
            break;
         case 1: /* verify the new balance (redundent) */
            /* attempt to read the first page */
            if (TMExtendedReadPage(session_handle,state_buffer,
                                   MoneyFile->startpage,read_buf,0) == 40)
            {
               /* check for valid data packet */
               if (VerifyCRCPacket(MoneyFile->startpage,read_buf))
               {
                  /* copy counter to end of file data */
                  for (i = 0; i < 4; i++)
                  {
                     read_buf[29 + i] = read_buf[32 + i];
                     MoneyFile->bitmap[i] = read_buf[29 + i];
                  }
                  /* create dummy tamper bytes */
                  for (i = 0; i < 4; i++)
                     read_buf[33 + i] = read_buf[36 + i];         
                  if (VerifySignedMoney(&balance, MoneyFile, &read_buf[1], 28))
                  {
                     if (balance == newbalance)
                     {
                        sprintf(LastStatus,"New money balance verified");
                        return TRUE;    
                     }
                     else if ((balance != oldbalance) || (read_buf[8] != HOST_ID))
                     {
                        sprintf(LastError,"During money update, tampering detected!");
                        sprintf(LastStatus,"%s",LastError);
                        return FALSE;
                     }
                  }
                  else
                  {
                     sprintf(LastError,"During money update, tampering detected!");
                     sprintf(LastStatus,"%s",LastError);
                     return FALSE;
                  }
               }
                 
               operation = 0;
            }                     
            else
               errorcnt++; 
            break;
      }

      /* check for an error, verify device is present */
      if (errorcnt > 0)      
      {
         rslt = TMStrongAccess(session_handle, state_buffer);
         if (rslt != 1)
         {
            if (printmsg)
            {
               printf("\nPlease reconnect the iButton to complete the operation!\n");
               printmsg = FALSE; 
            }
            errorcnt = 0; 
         }
      }
      else
         printmsg = TRUE;

      /* check for too many errors */
      if (errorcnt > 40)
      {
         sprintf(LastError,"Failed to complete the money deduction!\n");
         sprintf(LastStatus,"%s",LastError);
         return FALSE;
      } 
   }
   while (!InterruptOperation());

   sprintf(LastError,"Operation aborted\n");
   sprintf(LastStatus,"%s",LastError);

   return FALSE;
}


                           
