/*--------------------------------------------------------------------------
 * Copyright (C) 1992-1999 Dallas Semiconductor Corporation. 
 * All rights Reserved. Printed in U.S.A.
 * This software is protected by copyright laws of
 * the United States and of foreign countries.
 * This material may also be protected by patent laws of the United States 
 * and of foreign countries.
 * This software is furnished under a license agreement and/or a
 * nondisclosure agreement and may only be used or copied in accordance
 * with the terms of those agreements.
 * The mere transfer of this software does not imply any licenses
 * of trade secrets, proprietary technology, copyrights, patents,
 * trademarks, maskwork rights, or any other form of intellectual
 * property whatsoever. Dallas Semiconductor retains all ownership rights.
 *--------------------------------------------------------------------------
 *
 *  MonView.C - This utility uses TMEX to read the information and money
 *              file of a DS1963. 
 *
 *  Compiler:  Microsoft Visual C 
 *  Version:   1.00
 *
 */

#define TMEXUTIL

#include "ibtmex.h"   

/* external function prototypes */
extern short ReadDefaultPort(void);
extern void Exit_Prog(char *, short);
extern void PrintHeader(char *);
extern short Aquire1WireNet(void);
extern void Release1WireNet(void);
extern short WaitForiButton(uchar);
extern short MonetaryiButtonNotFormated(void);
extern short InterruptOperation(void);
extern short EnterNum(char *, short, ulong *);
extern short EnterStr(char *, short, char *);
extern short InitMonetaryiButton(ulong, char *);
extern short VerifySignedMoney(ulong *, FileEntry *, uchar *, short);
extern short ReadMoneyFiles(uchar *, short *, uchar *, short *, FileEntry *);

/*----------------------------------------------------------------------
 *  This is the Main routine for MonView.
 */
void main(short argc, char **argv)
{
   short money_len,info_len,st;
   ulong balance;
   uchar money_buf[50],info_buf[500];
   FileEntry MoneyFile;

   /* check arguments to see if request instruction with '?' or too many */
   if (argc > 1)
   {
       printf("\nusage: MonView\n"
              "  - view the info file and money balance\n"
              "  - version 1.00\n");
       Exit_Prog("",BAD_ARGUMENT);
   }

   /* read the default 1-Wire Net Port */
   if (!ReadDefaultPort())
     Exit_Prog("ERROR, Could not read the default PortNum and PortType from registry!\n",
               NO_DRIVERS);

   /* print the header info */
   PrintHeader("MonView Version 1.00");

   /* aquired the 1-Wire Net */
   printf("Aquire a session on the 1-Wire Net...\n");
   if (Aquire1WireNet())
   {
      printf("STATUS:%s\n",LastStatus);
      printf("\nWait for a Monetary iButton...\n");

      /* wait for a monetary iButton to arrive */
      if (WaitForiButton(MONETARY_FAM))
      {      
         printf("STATUS:%s\n",LastStatus);

         /* read both files */
         if (ReadMoneyFiles(info_buf, &info_len, money_buf, 
              &money_len, &MoneyFile))
         {
            printf("STATUS:%s\n",LastStatus);
            
            /* assume info is text, put in status */
            info_buf[info_len] = 0;
            st = sprintf(LastStatus,"\n\nInformation File Contents:\n%s\n\n",(char *)info_buf);

            st += sprintf(&LastStatus[st],"Money File Contents:\n");
            if (VerifySignedMoney(&balance, &MoneyFile, money_buf, money_len))
               st += sprintf(&LastStatus[st],"----- VERIFIED -----");
            else
               st += sprintf(&LastStatus[st],"?????? INVALID ?????");
            st += sprintf(&LastStatus[st],"\nBalance: %d\n\n",balance);
         }
         else
            sprintf(LastStatus,"Read aborted!\n");
      }
      else
        sprintf(LastStatus,"Wait for Monetary iButton aborted!\n");
   
   }
   else
     sprintf(LastStatus,"Wait for 1-Wire Net aborted!\n");

   /* print the last status */
   printf("STATUS:%s\n",LastStatus);

   /* release the 1-Wire Net to other processes */
   Release1WireNet();
   printf("STATUS:%s\n",LastStatus);

   /* normal exit */
   Exit_Prog("", NORMAL_EXIT);
}